using System;
using System.Collections.Generic;
using System.Linq;

using Syntriax.Engine.Core;
using Syntriax.Engine.Core.Serialization;

using YamlDotNet.Core;
using YamlDotNet.Core.Events;
using YamlDotNet.Serialization;

namespace Syntriax.Engine.Serializers.Yaml;

public class BehaviourControllerConverter : EngineTypeYamlSerializerBase<IBehaviourController>
{
    private const string BEHAVIOURS_SCALAR_NAME = "Behaviours";

    public override IBehaviourController? Read(IParser parser, Type type, ObjectDeserializer rootDeserializer)
    {
        string id;

        IBehaviourController behaviourController;
        IStateEnable stateEnable;
        List<IBehaviour> behaviours;

        parser.Consume<MappingStart>();

        if (parser.Consume<Scalar>().Value.CompareTo(nameof(IBehaviourController.Id)) != 0)
            throw new();
        id = parser.Consume<Scalar>().Value;

        if (parser.Consume<Scalar>().Value.CompareTo(SERIALIZED_SCALAR_NAME) != 0)
            throw new();
        SerializedClass instanceSerializedClass = (SerializedClass)rootDeserializer(typeof(SerializedClass))!;
        behaviourController = (IBehaviourController)instanceSerializedClass.CreateInstance(EntityRegistry);

        string value = parser.Consume<Scalar>().Value;
        if (value.CompareTo(nameof(IBehaviourController.StateEnable)) != 0)
            throw new();
        stateEnable = (IStateEnable)rootDeserializer(typeof(IStateEnable))!;

        if (parser.Consume<Scalar>().Value.CompareTo(BEHAVIOURS_SCALAR_NAME) != 0)
            throw new();
        behaviours = (List<IBehaviour>)rootDeserializer(typeof(List<IBehaviour>))!;

        parser.Consume<MappingEnd>();

        behaviourController.Id = id;

        stateEnable.Assign(behaviourController);
        behaviourController.Assign(stateEnable);

        foreach (IBehaviour behaviour in behaviours)
            behaviourController.AddBehaviour(behaviour);

        return behaviourController;
    }

    public override void WriteYaml(IEmitter emitter, object? value, Type type, ObjectSerializer serializer)
    {
        IBehaviourController behaviourController = (IBehaviourController)value!;

        emitter.Emit(new MappingStart());

        emitter.Emit(new Scalar(nameof(IBehaviourController.Id)));
        emitter.Emit(new Scalar(behaviourController.Id));

        emitter.Emit(new Scalar(SERIALIZED_SCALAR_NAME));
        serializer(new SerializedClass(behaviourController));

        emitter.Emit(new Scalar(nameof(IBehaviourController.StateEnable)));
        serializer(behaviourController.StateEnable);

        emitter.Emit(new Scalar(BEHAVIOURS_SCALAR_NAME));
        serializer(behaviourController.GetBehaviours<IBehaviour>().Where(b => !b.GetType().HasAttribute<IgnoreSerializationAttribute>()));

        emitter.Emit(new MappingEnd());
    }
}
